<?php

/**
 * Plugin Name: WooCommerce ForteEcom payment widget
 * Plugin URI:
 * Description: Take credit card payments on your store using ForteEcom.
 * Author: OneTech
 * Author URI: https://one.kz/
 * Version: 1.0.0
 * Requires at least: 4.4
 * Tested up to: 5.3.0
 * WC requires at least: 3.0
 * WC tested up to: 3.8
 * Text Domain: wc-gateway-forteecom
 * Domain Path: /languages/
 *
 */

/**
 * Disable gateway if Woocommerce is not installed or inactive
 */
defined('ABSPATH') or exit;
if (!in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {
    return;
}

/**
 * Adding ForteEcom gateway to WC gateways list.
 * To be able to handle multiple endpoints on merchant side, add dummy class
 *
 * @param $gateways
 * @return array
 */
function wc_forteecom_add_to_gateways($gateways)
{
    $gateways[] = 'WC_Gateway_ForteEcom';
    // $gateways[] = 'ForteEcom_Webhook';
    return $gateways;
}

add_filter('woocommerce_payment_gateways', 'wc_forteecom_add_to_gateways');

/**
 * Plugin configuration link on plugins page
 *
 * @param $links
 * @return array
 */
function wc_forteecom_gateway_plugin_links($links)
{
    $plugin_links = array(
        '<a href="' . admin_url('admin.php?page=wc-settings&tab=checkout&section=forteecom') . '">' . __('Configure', 'wc-gateway-forteecom') . '</a>'
    );
    return array_merge($plugin_links, $links);
}

add_filter('plugin_action_links_' . plugin_basename(__FILE__), 'wc_forteecom_gateway_plugin_links');

/**
 * Add gateway scripts
 */
function forteecom_add_scripts()
{
//    wp_register_script('forte-ecom', 'https://cdn-1.forte.kz/assets/plugins/forte-ecom/v1/ecom-widget.js', array(), time());
    wp_enqueue_script('forte-ecom');
}

add_action('wp_enqueue_scripts', 'forteecom_add_scripts');

/**
 * Init gateway after woocommerce inited (3rd param set to 11)
 */
add_action('plugins_loaded', 'wc_forteecom_gateway_init', 11);
function wc_forteecom_gateway_init()
{

    require_once dirname(__FILE__) . DIRECTORY_SEPARATOR . 'includes/classes/ForteEcom_Helper.php';
    require_once dirname(__FILE__) . DIRECTORY_SEPARATOR . 'includes/classes/WC_Gateway_ForteEcom.php';

    $plugin_dir = basename(dirname(__FILE__)) . '/languages';
    load_plugin_textdomain('wc-gateway-forteecom', false, $plugin_dir);

    // hook to cron
    if (!wp_next_scheduled('forteecom_update_pending_statuses_hook')) {
        wp_schedule_event(time(), 'hourly', 'forteecom_update_pending_statuses_hook');
    }
    add_action('forteecom_update_pending_statuses_hook', 'forteecom_update_pending_statuses');
}

/**
 * Disable cron task when plugin deactivated
 */
register_deactivation_hook(__FILE__, 'deactivate_forteecom_cron');
function deactivate_forteecom_cron()
{
    $timestamp = wp_next_scheduled('forteecom_update_pending_statuses_hook');
    wp_unschedule_event($timestamp, 'forteecom_update_pending_statuses_hook');
}

/**
 * add KZT to currencies
 */
add_filter('woocommerce_currencies', 'forteecom_add_kzt_currency');
function forteecom_add_kzt_currency($currencies)
{
    $currencies['KZT'] = __('Kazakhstani tenge', 'woocommerce');
    return $currencies;
}

/**
 * add KZT symbol
 */
add_filter('woocommerce_currency_symbol', 'forteecom_add_kzt_currency_symbol', 10, 2);
function forteecom_add_kzt_currency_symbol($currency_symbol, $currency)
{
    switch ($currency) {
        case 'KZT':
            $currency_symbol = '&#x20b8;';
            break;
    }
    return $currency_symbol;
}

/**
 * Runs by CRON. Updates statuses for on-hold orders
 * @noinspection SqlNoDataSourceInspection
 */
function forteecom_update_pending_statuses()
{
    global $wpdb;
    $orders = $wpdb->get_col("
            SELECT DISTINCT p.ID
            FROM 
                $wpdb->posts as p 
                LEFT JOIN $wpdb->postmeta as pm ON p.ID = pm.post_id 
            WHERE 
                p.post_status IN ('wc-pending', 'wc-on-hold') AND
                p.post_type = 'shop_order' AND  
                pm.meta_key = '_payment_method' AND 
                pm.meta_value = 'forteecom'");
    foreach ($orders as $order_id) {

        $_forteecom_order_status = get_post_meta($order_id, '_forteecom_order_status', true);
        $_forteecom_order_id     = get_post_meta($order_id, '_forteecom_order_id', true);
        $_forteecom_session_id   = get_post_meta($order_id, '_forteecom_session_id', true);
        $_forteecom_merchant_id  = get_post_meta($order_id, '_forteecom_merchant_id', true);

        if ($_forteecom_order_id && $_forteecom_session_id && $_forteecom_merchant_id && $_forteecom_order_status) {
            $order         = wc_get_order($order_id);
            $status_result = ForteEcom_Helper::get_payment_status($_forteecom_merchant_id, $_forteecom_order_id, $_forteecom_session_id);
            ForteEcom_Helper::update_payment_status($order, $order_id, $_forteecom_order_id, $status_result['_forteecom_order_status']);
        }
    }
}
