<?php
/**
 * Helper class
 */
class ForteEcom_Helper
{
	
    const HOST = "https://qa-merchant-go.fortebank.com";
    const HOST_DEMO = "https://qa-merchant-go.fortebank.com"; 
	
	
	/*
	const HOST = "https://qa-merchant-go.forte.kz";
    const HOST_DEMO = "https://qa-merchant-go.forte.kz";
	*/
    const ENDPOINT_INIT = "/init";
    const ENDPOINT_STATUS = "/status";
    const ENDPOINT_COMPLETE = "/complete";
    const ENDPOINT_PURCHASE = "/purchase";
    const ENDPOINT_PURCHASE_TOKEN = "/token/purchase";
    const ENDPOINT_REFUND = "/refund";
    const ENDPOINT_REVERSE = "/reverse";

    const URL_RESPONSE_CODE = "https://adminapi.forte.kz/codes";

    const STATUS_INIT = 'CREATED';
    const STATUS_LOCK = 'ON_LOCK';
    const STATUS_PROCESS = 'ON_PAYMENT';
    const STATUS_SUCCESS = 'APPROVED';
    const STATUS_DECLINE = 'DECLINED';
    const STATUS_ON_HOLD = 'PREAUTH_APPROVED';
    const STATUS_REVERSE = 'REVERSED';
    const STATUS_REFUND = 'REFUNDED';
    const STATUS_EXPIRE = 'EXPIRED';
    const STATUS_CANCEL = 'CANCELED';
    const STATUS_ERROR = 'ERROR';

    const WC_LOG_FILENAME = 'woocommerce-gateway-forteecom';

    /**
     * Gets ForteEcom payment status
     *
     * @param string $_forteecom_merchant_id
     * @param string $_forteecom_order_id
     * @param string $_forteecom_session_id
     *
     * @return array
     */
    public static function get_payment_status($_forteecom_merchant_id, $_forteecom_order_id, $_forteecom_session_id)
    {
        $data = array(
            'merchantId' => $_forteecom_merchant_id,
            'orderId'    => $_forteecom_order_id,
            'sessionId'  => $_forteecom_session_id,
        );

        $request_body = json_encode($data);

        $request_result = self::send_request(self::ENDPOINT_STATUS, $request_body);

        $_forteecom_order_status = null;
        $_forteecom_add_info     = null;
        if (empty($request_result['error'])) {
            $_forteecom_order_status = $request_result['response']->data->orderStatus;
            $_forteecom_add_info     = $request_result['response']->data->addInfo;
        }

        return array(
            'error_message'           => $request_result['error'],
            '_forteecom_order_status' => $_forteecom_order_status,
            '_forteecom_add_info'     => $_forteecom_add_info,
        );
    }

    /**
     * @param string $endpoint
     * @param        $body
     *
     * @return array
     */
    public static function send_request($endpoint, $body)
    {
        $curl = curl_init();
        $url = self::get_url($endpoint);
        curl_setopt_array($curl, array(
            CURLOPT_URL            => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_SSL_VERIFYHOST => 0,
            CURLOPT_SSL_VERIFYPEER => 0,
            CURLOPT_ENCODING       => '',
            CURLOPT_MAXREDIRS      => 10,
            CURLOPT_TIMEOUT        => 30,
            CURLOPT_HTTP_VERSION   => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST  => 'POST',
            CURLOPT_POSTFIELDS     => $body,
            CURLOPT_HTTPHEADER     => array(
                'Content-Type: application/json'
            )
        ));
        $response = json_decode(curl_exec($curl));
		//$response = (curl_exec($curl));
		//print_r($curl);
		//echo "<br>";
		//print_r($url);
		//echo "<br>";
		//var_dump($response);
        $error    = curl_error($curl);
        curl_close($curl);

        if (empty($error) && !property_exists($response, 'responseCode')) {
            $error = $response->moreInfo;
        }

        $log_data = array(
            'url'      => $url,
            'request'  => $body,
            'response' => $response,
            'error'    => $error,
        );
        self::log(print_r($log_data, true));

        return array(
            'response' => $response,
            'error'    => $error
        );
    }

    public static function get_url($endpoint)
    {
        $host = (FORTEECOM_DEMO_ENABLED) ? self::HOST_DEMO : self::HOST;
        return $host . '/ecom' . $endpoint;
    }

    /**
     * Log message
     *
     * @param string $message
     */
    public static function log($message)
    {
        if (!class_exists('WC_Logger') || !FORTEECOM_LOGGING_ENABLED) {
            return;
        }

        if (self::is_wc_lt('3.0')) {
            $logger = new WC_Logger();
        } else {
            $logger = wc_get_logger();
        }

        $log_entry = "\n" . '====Forteecom Log====' . "\n";
        $log_entry .= '====Start====' . "\n" . $message . "\n" . '====End====' . "\n\n";

        if (self::is_wc_lt('3.0')) {
            $logger->add(self::WC_LOG_FILENAME, $log_entry);
        } else {
            $logger->debug($log_entry, array('source' => self::WC_LOG_FILENAME));
        }
    }

    /**
     * Checks if WooCommerce is less than specified version
     *
     * @param string $version
     *
     * @return bool|int
     */
    public static function is_wc_lt($version)
    {
        return version_compare(WC_VERSION, $version, '<');
    }

    /**
     * Completes ForteEcom "pre-auth"-ed payment
     *
     * @param string $_forteecom_merchant_id
     * @param string $_forteecom_order_id
     * @param string $_forteecom_session_id
     *
     * @return array
     */
    public static function complete_payment($_forteecom_merchant_id, $_forteecom_order_id, $_forteecom_session_id)
    {
        $data = array(
            'merchantId' => $_forteecom_merchant_id,
            'orderId'    => $_forteecom_order_id,
            'sessionId'  => $_forteecom_session_id,
        );

        $request_body = json_encode($data);

        $request_result = self::send_request(self::ENDPOINT_COMPLETE, $request_body);

        $_forteecom_approval_code = null;
        if (empty($request_result['error'])) {
            $_forteecom_approval_code = $request_result['response']->data->approvalCode;
        }

        return array(
            'error_message'            => $request_result['error'],
            '_forteecom_approval_code' => $_forteecom_approval_code,
        );
    }

    public static function refund_payment($_forteecom_merchant_id, $_forteecom_order_id, $_forteecom_session_id, $amount, $currency_id)
    {
        $data        = array(
            'orderIdent' => array(
                'merchantId' => $_forteecom_merchant_id,
                'orderId'    => $_forteecom_order_id,
                'sessionId'  => $_forteecom_session_id,
            ),
            'transaction' => array(
                'jsonClass'   => "TransactionSourceRefund",
                'amount'      => (float)$amount,
                'currency'    => (string)$currency_id
            ),
        );

        $request_body = json_encode($data);

        $request_result = self::send_request(self::ENDPOINT_REFUND, $request_body);

        $_forteecom_message = null;
        if (empty($request_result['error']) && $request_result['response']->message == 'OK') {
            $_forteecom_message = $request_result['response']->message;
        } else {
            $request_result['error'] = 'Error occured on refund';
        }

        return [
            'error_message'      => $request_result['error'],
            '_forteecom_message' => $_forteecom_message,
        ];
    }

    /**
     * @param string              $_forteecom_merchant_id
     * @param string              $_forteecom_order_id
     * @param string              $_forteecom_session_id
     * @param float               $amount
     * @param string                 $remote_id
     * @param string              $description
     * @param string                 $currency_id
     * @param WC_Payment_Token_CC $card
     *
     * @return array
     */
    public static function purchase_by_token($_forteecom_merchant_id, $_forteecom_order_id, $_forteecom_session_id, $amount, $remote_id, $description, $currency_id, $card)
    {

        $data = array(
            'merchantId' => $_forteecom_merchant_id,
            'orderId'    => $_forteecom_order_id,
            'sessionId'  => $_forteecom_session_id,
            'ip'         => $_SERVER['REMOTE_ADDR'],
            'token'      => $card->get_token(),
            'source'     => array(
                'jsonClass'   => "TransactionSourcePurchase",
                'amount'      => (float)$amount,
                'currency'    => (string)$currency_id,
                'remoteId'    => (string)$remote_id,
                'description' => $description
            ),
        );

        $request_body   = json_encode($data);
        $request_result = self::send_request(self::ENDPOINT_PURCHASE_TOKEN, $request_body);

        $_forteecom_order_status = null;
        $_forteecom_order_data   = null;
        if (empty($request_result['error'])) {
            $_forteecom_order_status = $request_result['response']->responseMessage;
            if ($request_result['response']->responseCode != 0) {
                $_forteecom_order_data   = $request_result['response']->data->purchaseResponseCode;
                $request_result['error'] = self::get_response_message($request_result['response']->data->purchaseResponseCode);
            } else {
                $_forteecom_order_data = $request_result['response']->data->result;
            }
        }

        return array(
            'error_message'           => $request_result['error'],
            '_forteecom_order_status' => $_forteecom_order_status,
            '_forteecom_order_data'   => $_forteecom_order_data,
        );

    }

    /**
     * Get localized message for error
     *
     * @param string $code
     *
     * @return string
     */
    public static function get_response_message($code)
    {

        switch (get_locale()) {
            case 'ru_RU':
                $lang = '_ru';
                break;
            case 'kk':
                $lang = '_kk';
                break;
            default:
                $lang = '';
                break;
        }

        $url = self::URL_RESPONSE_CODE . '?' . http_build_query(['name' => $code]);

        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL            => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_SSL_VERIFYHOST => 0,
            CURLOPT_SSL_VERIFYPEER => 0,
            CURLOPT_ENCODING       => '',
            CURLOPT_MAXREDIRS      => 10,
            CURLOPT_TIMEOUT        => 30,
            CURLOPT_HTTP_VERSION   => CURL_HTTP_VERSION_1_1
        ));

        $response = json_decode(curl_exec($curl), true);
        $error    = curl_error($curl);
        curl_close($curl);

        return $response[0]['message' . $lang];
    }

    /**
     * Initialize ForteEcom payment
     *
     * @param string $_forteecom_merchant_id
     * @param bool   $pre_auth
     * @param float  $amount
     * @param string $remote_id   Order id on merchant side
     * @param string $description Order description on merchant side
     * @param string $currency_id
     *
     * @return array
     */
    public static function init_payment($_forteecom_merchant_id, $pre_auth, $amount, $remote_id, $description, $currency_id)
    {
        $data = array(
            'merchantId' => $_forteecom_merchant_id,
            'preAuth'    => $pre_auth,
            'source'     => array(
                'jsonClass'   => 'TransactionSourcePurchase',
                'amount'      => (float)$amount,
                'currency'    => (string)$currency_id,
                'remoteId'    => (string) $remote_id,
                'description' => $description
            )
        );

        $request_body = json_encode($data);

        $request_result = self::send_request(self::ENDPOINT_INIT, $request_body);
		//echo "<br>";
		//echo "OK";
		//echo "<br>";
		//var_dump(self::ENDPOINT_INIT);
		//echo "<br>";
		//var_dump($request_body);
		//echo "<br>";
		//var_dump($request_result);
		//echo "<br>";
		//var_dump($request_result['response']->data->orderId);
		//echo "<br>";
		//var_dump($request_result['response']->data->sessionId);
		//echo "<br>";
		
		//die;
        $_forteecom_order_id   = null;
        $_forteecom_session_id = null;
		
		
        if (empty($request_result['error'])) {
			
            if ($request_result['response']->responseCode == 0) {
				
                $_forteecom_order_id   = $request_result['response']->data->orderId;
                $_forteecom_session_id = $request_result['response']->data->sessionId;
            } else {
                $request_result['error'] = $request_result['response']->responseMessage;
            }
        }
		
        return array(
            'error_message'         => $request_result['error'],
            '_forteecom_order_id'   => $_forteecom_order_id,
            '_forteecom_session_id' => $_forteecom_session_id,
        );
    }

    /**
     * Updates order data based on ForteEcom status
     *
     * @param WC_Order $order
     * @param int      $order_id   Woocommerce order id
     * @param string   $_forteecom_order_id
     * @param string   $_forteecom_order_status
     * @param bool     $empty_cart Do we need to remove products from cart
     */
    public
    static function update_payment_status(
        $order, $order_id, $_forteecom_order_id, $_forteecom_order_status
    )
    {
        global $woocommerce;
        $current_status = get_post_meta($order_id, '_forteecom_order_status', true);
        if ($current_status == $_forteecom_order_status) {
            return;
        }
        update_post_meta($order_id, '_forteecom_order_status', $_forteecom_order_status);
        switch ($_forteecom_order_status) {
            case self::STATUS_SUCCESS:
                self::is_wc_lt('3.0') ? $order->reduce_order_stock() : wc_reduce_stock_levels($order_id);
                $order->payment_complete($_forteecom_order_id);
                break;
            case self::STATUS_DECLINE:
                $order->update_status('failed');
                $order->add_order_note(__('Your payment failed, please retry.', 'wc-gateway-forteecom'), true);
                break;
            case self::STATUS_EXPIRE:
                $order->update_status('failed');
                $order->add_order_note(__('Payment expired', 'wc-gateway-forteecom'));
                break;
            case self::STATUS_REVERSE:
                $order->update_status('refunded');
                $order->add_order_note(__('Payment refunded', 'wc-gateway-forteecom'));
                break;
            case self::STATUS_REFUND:
                $order->update_status('refunded');
                $order->add_order_note(__('Payment reversed', 'wc-gateway-forteecom'));
                break;
            case self::STATUS_CANCEL:
                $order->update_status('cancelled');
                $order->add_order_note(__('Payment cancelled by customer', 'wc-gateway-forteecom'));
                break;
            case self::STATUS_ON_HOLD:
                self::is_wc_lt('3.0') ? $order->reduce_order_stock() : wc_reduce_stock_levels($order_id);
                $order->update_status('on-hold');
                $order->add_order_note(__('Your order is placed and payment is on hold.', 'wc-gateway-forteecom'), true);
                break;
        }
    }

    /**
     * Process order data based on ForteEcom status
     *
     * @param string $_forteecom_order_status
     */
    public
    static function process_payment_status(
        $_forteecom_order_status
    )
    {
        global $woocommerce;

        switch ($_forteecom_order_status) {
            case self::STATUS_ON_HOLD:
            case self::STATUS_SUCCESS:
                $woocommerce->cart->empty_cart();
                break;
        }
    }

    /**
     * @param string $_forteecom_order_id
     *
     * @return bool|string|null
     */
    public
    static function get_order_id_by_forteecom_order_id(
        $_forteecom_order_id
    )
    {
        global $wpdb;

        if (empty($_forteecom_order_id)) {
            return false;
        }

        $order_id = $wpdb->get_var($wpdb->prepare("SELECT DISTINCT ID FROM $wpdb->posts as posts LEFT JOIN $wpdb->postmeta as meta ON posts.ID = meta.post_id WHERE meta.meta_value = %s AND meta.meta_key = %s", $_forteecom_order_id, '_forteecom_order_id'));

        if (!empty($order_id)) {
            return $order_id;
        }

        return false;
    }

    public static function get_script() {
        if (FORTEECOM_DEMO_ENABLED) {
            $cdn_url = 'https://cdn-1.forte.kz/assets/plugins/forte-ecom-qago/v1/ecom-widget-feature-plugin/';
        } else {
            $cdn_url = 'https://cdn-1.forte.kz/assets/plugins/forte-ecom-qago/v1/ecom-widget-feature-plugin/';
        }

        return '<script type="module" src="' . $cdn_url . 'ecom-widget-feature-plugin.esm.js?ver=' . time() . '"></script>
        <script nomodule="" src="' . $cdn_url . 'ecom-widget-feature-plugin.js?ver=' . time() . '"></script>';
    }

}
